@@ -11,8 +11,8 @@
     s3_register("base::print", "date_names")
     s3_register("base::print", "locale")
     s3_register("utils::str", "col_spec")
-    s3_register("testthat::compare", "tbl_df")
-    s3_register("waldo::compare_proxy", "tbl_df")
+    s3_register("testthat::compare", "spec_tbl_df")
+    s3_register("waldo::compare_proxy", "spec_tbl_df")
   }
 }
 
