@@ -29,7 +29,8 @@ delimited_index_connection::delimited_index_connection(
     const bool has_header,
     const size_t skip,
     size_t n_max,
-    const char comment,
+    const char* comment,
+    const std::shared_ptr<vroom_errors> errors,
     const size_t chunk_size,
     const bool progress) {
 
@@ -148,7 +149,10 @@ delimited_index_connection::delimited_index_connection(
       n_max,
       cols,
       0,
-      empty_pb);
+      errors,
+      empty_pb,
+      1,
+      -1);
 
   columns_ = idx_[0].size() - 1;
 
@@ -183,7 +187,10 @@ delimited_index_connection::delimited_index_connection(
             n_max,
             cols,
             columns_,
-            empty_pb);
+            errors,
+            empty_pb,
+            1,
+            -1);
       });
     }
 
