@@ -16,7 +16,8 @@ public:
       const bool has_header,
       const size_t skip,
       const size_t n_max,
-      const char comment,
+      const char* comment,
+      const std::shared_ptr<vroom_errors> errors,
       const size_t chunk_size,
       const bool progress);
 
