@@ -42,6 +42,7 @@ protected:
   mio::mmap_source mmap_;
   bool trim_ws_;
   bool windows_newlines_;
+  std::string filename_;
 
 public:
   fixed_width_index(
@@ -50,10 +51,13 @@ public:
       std::vector<int> col_ends,
       bool trim_ws,
       const size_t skip,
-      const char comment,
+      const char* comment,
       const size_t n_max,
       const bool progress)
-      : col_starts_(col_starts), col_ends_(col_ends), trim_ws_(trim_ws) {
+      : col_starts_(col_starts),
+        col_ends_(col_ends),
+        trim_ws_(trim_ws),
+        filename_(filename) {
 
     std::error_code error;
     mmap_ = make_mmap_source(filename, error);
@@ -169,6 +173,9 @@ public:
     string value() const { return idx_->get(i_, column_); }
     column_iterator* clone() const { return new column_iterator(*this); }
     string at(ptrdiff_t n) const { return idx_->get(n, column_); }
+    std::string filename() const { return idx_->filename_; }
+    size_t index() const { return i_ / idx_->num_columns(); }
+    size_t position() const { return i_; }
     virtual ~column_iterator() = default;
   };
 
@@ -177,7 +184,7 @@ public:
     auto end = new column_iterator(shared_from_this(), column);
     end->advance(num_rows());
 
-    return std::make_shared<vroom::index::column>(begin, end);
+    return std::make_shared<vroom::index::column>(begin, end, column);
   }
 
   template <typename T>
@@ -230,5 +237,7 @@ public:
     // TODO: UNUSED
     return nullptr;
   }
+
+  std::string filename() const { return filename_; }
 };
 } // namespace vroom
