@@ -6,6 +6,7 @@
 #include "LocaleInfo.h"
 #include "altrep.h"
 #include "index_collection.h"
+#include "vroom_errors.h"
 
 using namespace vroom;
 
@@ -14,9 +15,47 @@ struct vroom_vec_info {
   size_t num_threads;
   std::shared_ptr<cpp11::strings> na;
   std::shared_ptr<LocaleInfo> locale;
+  std::shared_ptr<vroom_errors> errors;
   std::string format;
 };
 
+namespace vroom {
+
+inline bool is_explicit_na(SEXP na, const char* begin, const char* end) {
+  R_xlen_t n = end - begin;
+  for (R_xlen_t i = 0; i < Rf_xlength(na); ++i) {
+    SEXP str = STRING_ELT(na, i);
+    R_xlen_t str_n = Rf_xlength(str);
+    const char* v = CHAR(STRING_ELT(na, i));
+    if (n == str_n && strncmp(v, begin, n) == 0) {
+      return true;
+    }
+  }
+  return false;
+}
+
+template <typename V, typename F, typename I, typename C>
+static auto parse_value(
+    I itr,
+    C col,
+    F f,
+    std::shared_ptr<vroom_errors>& errors,
+    const char* expected,
+    SEXP na) -> V {
+  auto str = *itr;
+  V out = f(str.begin(), str.end());
+  if (cpp11::is_na(out) && !is_explicit_na(na, str.begin(), str.end())) {
+    errors->add_error(
+        itr.index(),
+        col->get_index(),
+        expected,
+        std::string(str.begin(), str.end() - str.begin()),
+        itr.filename());
+  }
+  return out;
+}
+} // namespace vroom
+
 #ifdef HAS_ALTREP
 
 class vroom_vec {
@@ -65,7 +104,8 @@ public:
     return STDVEC_DATAPTR(data2);
   }
 
-  static std::shared_ptr<std::vector<size_t>> get_subset_index(SEXP indx) {
+  static std::shared_ptr<std::vector<size_t>>
+  get_subset_index(SEXP indx, R_xlen_t x_len) {
     auto idx = std::make_shared<std::vector<size_t>>();
     R_xlen_t n = Rf_xlength(indx);
     idx->reserve(n);
@@ -77,14 +117,14 @@ public:
       switch (TYPEOF(indx)) {
       case INTSXP:
         i_val = INTEGER_ELT(indx, i);
-        if (i_val == NA_INTEGER) {
+        if (i_val == NA_INTEGER || i_val > x_len) {
           return nullptr;
         }
         idx->push_back(i_val - 1);
         break;
       case REALSXP:
         d_val = REAL_ELT(indx, i);
-        if (ISNA(d_val)) {
+        if (ISNA(d_val) || d_val > x_len) {
           return nullptr;
         }
         idx->push_back(d_val - 1);
@@ -115,7 +155,7 @@ public:
     {
       auto& inf = Info(x);
 
-      auto idx = get_subset_index(indx);
+      auto idx = get_subset_index(indx, Rf_xlength(x));
 
       if (idx == nullptr) {
         return nullptr;
@@ -126,6 +166,7 @@ public:
           inf.num_threads,
           inf.na,
           inf.locale,
+          inf.errors,
           inf.format};
     }
 
